/*****************************************************************************
 * aout_spdif: ac3 passthrough output
 *****************************************************************************
 * Copyright (C) 2001 VideoLAN
 * $Id: aout_spdif.c,v 1.2 2001/05/06 04:32:02 sam Exp $
 *
 * Authors: Michel Kaempf <maxx@via.ecp.fr>
 *          Stphane Borel <stef@via.ecp.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

/*****************************************************************************
 * Preamble
 *****************************************************************************/
#include "defs.h"

#include <stdio.h>                                           /* "intf_msg.h" */
#include <stdlib.h>                            /* calloc(), malloc(), free() */
#include <string.h>                                              /* memset() */

#include "config.h"
#include "common.h"
#include "threads.h"
#include "mtime.h"                             /* mtime_t, mdate(), msleep() */

#include "intf_msg.h"                        /* intf_DbgMsg(), intf_ErrMsg() */

#include "audio_output.h"
#include "aout_common.h"

/* temporary blank until I've found how to build
 * a real blank iec958 data frame */
static u8 pi_spdif_blank [] =
{
    0x72, 0xf8, 0x1f, 0x4e, 0x01, 0x00, 0x00, 0x30, 
    0x77, 0x0b, 0x07, 0x04, 0x40, 0x1c, 0xfc, 0xe1, 
    0xcb, 0x01, 0x7d, 0x80, 0xb2, 0xfe, 0x40, 0xe8, 
    0x06, 0x75, 0x84, 0x07, 0x14, 0x04, 0x0e, 0x14, 
    0x84, 0x83, 0x56, 0xd5, 0xbe, 0x52, 0xad, 0x5b, 
    0x27, 0xf4, 0x9f, 0xcf, 0xdf, 0x3f, 0x95, 0x6a, 
    0x86, 0x3d, 0xba, 0xa6, 0xb0, 0x67, 0xbe, 0xa1, 
    0xfd, 0x52, 0x9a, 0xd2, 0xb5, 0xcf, 0x7d, 0x5a, 
    0xf2, 0x4d, 0x4f, 0x5a, 0xf6, 0xbd, 0xfa, 0x95, 
    0xe9, 0x57, 0x3e, 0x95, 0xe9, 0x54, 0x77, 0x4d, 
    0x9e, 0x4a, 0x54, 0xd7, 0x15, 0xf6, 0xb0, 0x77, 
    0x3e, 0xdd, 0xab, 0x4c, 0xcf, 0xde, 0xc2, 0x6b, 
    0x96, 0x86, 0xf5, 0x93, 0xa1, 0x35, 0xad, 0x56, 
    0xeb, 0x5d, 0x4f, 0xea, 0xa6, 0x9d, 0xfa, 0x4a, 
    0xa7, 0x13, 0x7f, 0xe8, 0xfd, 0xb8, 0xf2, 0xf3, 
    0x9e, 0x2f, 0x73, 0xd7, 0xeb, 0x4d, 0xce, 0xf7, 
    0xa2, 0xa9, 0x0d, 0x49, 0xe7, 0x53, 0x9e, 0xcf, 
    0x7b, 0xd7, 0xfb, 0x5f, 0xab, 0x95, 0x54, 0x43, 
    0xbb, 0xf6, 0xd5, 0x9a, 0x3e, 0xdf, 0x5e, 0x73, 
    0xa7, 0x92, 0x9e, 0xca, 0x86, 0xd6, 0x33, 0xf9, 
    0x2f, 0xa7, 0xef, 0xfe, 0x28, 0xef, 0xfc, 0x3f, 
    0xf3, 0xf9, 0xcf, 0xe7, 0x20, 0x9f, 0xa8, 0x07, 
    0x00, 0x63, 0xb6, 0xd6, 0x6d, 0x0d, 0xe7, 0xa0, 
    0xd0, 0xc1, 0x1a, 0xb1, 0x35, 0xd8, 0x23, 0x47, 
    0xa5, 0x62, 0x96, 0x41, 0xd7, 0x9c, 0xe3, 0x59, 
    0x60, 0x9b, 0x15, 0xdb, 0xab, 0x03, 0x6a, 0x73, 
    0xc5, 0xd6, 0x5b, 0x8d, 0xe6, 0x5a, 0xed, 0xc0, 
    0x6c, 0x23, 0x2a, 0xb9, 0x65, 0xa7, 0xd6, 0x54, 
    0xb6, 0xb1, 0x15, 0x6c, 0x31, 0x55, 0x6d, 0xad, 
    0x07, 0x97, 0xb3, 0x11, 0x81, 0xb6, 0x14, 0x71, 
    0xd4, 0xe6, 0xda, 0xb6, 0xa8, 0x35, 0x09, 0x36, 
    0xe2, 0x85, 0x11, 0x45, 0x83, 0xad, 0x6a, 0x32, 
    0x67, 0xe1, 0xad, 0x35, 0x1b, 0xc9, 0x5a, 0x9b, 
    0x60, 0x06, 0x00, 0xf8, 0x05, 0x00, 0x00, 0x3c, 
    0x26, 0x12, 0x72, 0x05, 0xad, 0x94, 0x14, 0x8e, 
    0x07, 0x5e, 0xfb, 0x13, 0x91, 0x53, 0x83, 0x71, 
    0xad, 0x52, 0x0a, 0x87, 0xba, 0x3a, 0xf7, 0x38, 
    0x1a, 0xef, 0xa4, 0x65, 0x2e, 0x4b, 0x75, 0xeb, 
    0x52, 0x32, 0xc4, 0xaa, 0xe8, 0x83, 0xf8, 0x85, 
    0x51, 0x1d, 0x6f, 0x59, 0xea, 0xe3, 0x85, 0x07, 
    0x19, 0xb1, 0x9f, 0x6e, 0x36, 0x6a, 0x1a, 0x71, 
    0xa7, 0xec, 0x56, 0xed, 0xc8, 0x1b, 0xb6, 0xc3, 
    0xfd, 0xd0, 0x98, 0xd2, 0xa3, 0xf3, 0x5a, 0xfc, 
    0x1c, 0x5d, 0x60, 0x21, 0xc7, 0xf0, 0x08, 0x1e, 
    0x03, 0x90, 0xb1, 0xd4, 0x31, 0x18, 0xbc, 0x27, 
    0xb1, 0x6c, 0x8a, 0xf3, 0xe5, 0x68, 0x7a, 0x18, 
    0x29, 0xf0, 0xeb, 0x78, 0xdf, 0xe1, 0x3f, 0xf3, 
    0x67, 0x1b, 0xd4, 0x67, 0x20, 0x80, 0x0b, 0xb5, 
    0x6e, 0xef, 0x45, 0xbd, 0xac, 0x48, 0xe0, 0x5e, 
    0x3c, 0x6b, 0x3d, 0x7a, 0x7f, 0x38, 0xbd, 0xf1, 
    0xd3, 0x8f, 0xbf, 0xa1, 0x98, 0x50, 0xa0, 0x6a, 
    0x33, 0x5a, 0xc6, 0x2e, 0xfc, 0x8f, 0x8f, 0xcc, 
    0x58, 0xcf, 0x57, 0x22, 0xa6, 0xf7, 0x82, 0x14, 
    0x47, 0xea, 0x70, 0x81, 0x89, 0x86, 0x29, 0xb8, 
    0x83, 0xae, 0xd8, 0x5a, 0x4d, 0xb7, 0x0a, 0x24, 
    0xb5, 0xd6, 0x63, 0x4d, 0x46, 0x58, 0x79, 0xc5, 
    0x5b, 0x66, 0x2a, 0x68, 0x5b, 0x6b, 0xb9, 0x63, 
    0x6d, 0xad, 0x64, 0x92, 0xb1, 0xdb, 0xb3, 0xad, 
    0x46, 0x58, 0xcd, 0xb6, 0x48, 0x73, 0x35, 0xd6, 
    0x85, 0xc8, 0xf7, 0x68, 0xed, 0x88, 0x1a, 0x8b, 
    0x35, 0xd6, 0x23, 0xad, 0x06, 0x9b, 0x80, 0x91, 
    0x14, 0x23, 0x5a, 0x1c, 0xa9, 0x46, 0x4d, 0xb9, 
    0x8e, 0x6c, 0xa3, 0xc3, 0x5a, 0x0b, 0x5d, 0xa6, 
    0x6b, 0xad, 0xdb, 0x60, 0x26, 0x24, 0x8a, 0xb4, 
    0x20, 0x69, 0xb0, 0xd6, 0x1b, 0x2c, 0xa2, 0x60, 
    0x80, 0x0f, 0x00, 0x00, 0xe0, 0x53, 0xb4, 0x09, 
    0x0a, 0x78, 0xe4, 0xff, 0x13, 0x71, 0x56, 0xfd, 
    0x41, 0x08, 0xd7, 0x97, 0xb1, 0x43, 0x76, 0xe5, 
    0x02, 0x29, 0x40, 0xdc, 0xb2, 0xbe, 0x24, 0x72, 
    0x7e, 0xca, 0x30, 0x9d, 0x25, 0x92, 0x51, 0x73, 
    0x24, 0xfc, 0x96, 0xca, 0x5a, 0xee, 0xed, 0xf8, 
    0x65, 0x8e, 0x83, 0x94, 0xc2, 0xc2, 0x27, 0xd3, 
    0x50, 0xb1, 0xe7, 0xca, 0xcd, 0x1f, 0xd0, 0x51, 
    0x96, 0x34, 0xd0, 0x9d, 0x6a, 0xf5, 0x17, 0x45, 
    0x66, 0x48, 0x51, 0xdd, 0x69, 0x53, 0xac, 0xdc, 
    0xb1, 0xfc, 0x38, 0x79, 0x1b, 0x2a, 0xc1, 0xc8, 
    0x66, 0x01, 0xb5, 0xa1, 0x10, 0x89, 0x83, 0x59, 
    0xd6, 0xa6, 0x99, 0x66, 0x97, 0x60, 0xa4, 0x5e, 
    0x90, 0xdc, 0xf3, 0xb7, 0xa0, 0xc0, 0x54, 0xc8, 
    0xc5, 0x72, 0xf4, 0x65, 0x3c, 0xb9, 0x82, 0xa6, 
    0xa4, 0x99, 0x4b, 0xa0, 0x3b, 0xfa, 0x3f, 0x29, 
    0xfe, 0x00, 0x29, 0x77, 0xcc, 0xd3, 0x7a, 0x20, 
    0xc2, 0x38, 0xbe, 0xba, 0xdf, 0x35, 0x03, 0xbc, 
    0x01, 0xd1, 0x99, 0x5a, 0x65, 0x10, 0x2b, 0xac, 
    0xca, 0xf9, 0x0a, 0x87, 0x6e, 0x73, 0xe2, 0x20, 
    0x2e, 0x83, 0xe2, 0x4f, 0x32, 0x43, 0x6d, 0xb3, 
    0xb5, 0x36, 0x6b, 0xa5, 0xd6, 0x5a, 0xaa, 0xdd, 
    0x48, 0x6b, 0xb4, 0x76, 0x70, 0xea, 0x02, 0x63, 
    0xb6, 0x69, 0x06, 0xdb, 0x83, 0xb8, 0x6c, 0x60, 
    0x91, 0xa6, 0xa1, 0x47, 0x5a, 0x6b, 0x39, 0xdb, 
    0x69, 0xad, 0xd6, 0x18, 0x4a, 0xa9, 0x6c, 0x23, 
    0x88, 0x84, 0x36, 0xef, 0xe5, 0xda, 0xca, 0xba, 
    0x42, 0x70, 0xb1, 0xd6, 0x69, 0x8d, 0xc8, 0xda, 
    0x99, 0x29, 0x2e, 0xb7, 0x5c, 0x2e, 0xdd, 0xc6, 
    0x22, 0x99, 0x28, 0x5e, 0x4a, 0x41, 0x88, 0x6a, 
    0xa8, 0xe6, 0x7a, 0x90, 0x6a, 0x78, 0xd6, 0x9a, 
    0x84, 0x35, 0x86, 0x83, 0x36, 0x76, 0x40, 0x04, 
    0x00, 0xf8, 0x05, 0x00, 0x00, 0x40, 0x92, 0xee, 
    0x7b, 0xa1, 0xc8, 0xcf, 0x0b, 0x55, 0x6b, 0x30, 
    0xb5, 0x7b, 0x55, 0x72, 0x58, 0x74, 0x69, 0xf9, 
    0xd6, 0x9f, 0x89, 0x21, 0x0d, 0x5b, 0x90, 0x48, 
    0xde, 0x08, 0x65, 0xd4, 0x2b, 0x08, 0xe1, 0x22, 
    0xd0, 0xbd, 0xe9, 0x93, 0xb1, 0x80, 0x59, 0x83, 
    0xaa, 0xb7, 0x13, 0x57, 0x9f, 0xbf, 0xf8, 0x6c, 
    0x6d, 0xf9, 0xe6, 0x22, 0xf5, 0xc7, 0xc0, 0xdc, 
    0x17, 0x84, 0x63, 0xf8, 0xb3, 0x94, 0xb1, 0xd5, 
    0xa9, 0x56, 0x87, 0xd4, 0xd8, 0x2a, 0xc8, 0x56, 
    0x23, 0x70, 0x52, 0xb8, 0x66, 0x8b, 0x06, 0x5f, 
    0x84, 0xd5, 0xc6, 0xa6, 0x5a, 0xb1, 0x6f, 0xef, 
    0xcd, 0x7f, 0xe2, 0x9a, 0xb9, 0x9f, 0xff, 0x08, 
    0xff, 0x73, 0xa7, 0x99, 0x7f, 0x57, 0xb7, 0xa8, 
    0x19, 0xad, 0x11, 0x9c, 0xad, 0xd5, 0x97, 0xd3, 
    0x1a, 0x2e, 0x25, 0xa9, 0x42, 0x1d, 0x9e, 0x82, 
    0x05, 0xfa, 0xe0, 0xbf, 0x40, 0x2b, 0xce, 0xba, 
    0xd7, 0x0d, 0x44, 0xdb, 0xa7, 0x00, 0x27, 0x7d, 
    0x91, 0x66, 0x51, 0xfe, 0x75, 0xa0, 0xa4, 0xc4, 
    0x33, 0x0b, 0x04, 0x2c, 0xa8, 0x07, 0x4c, 0x94, 
    0xba, 0xf4, 0x60, 0x61, 0x41, 0x34, 0xb6, 0xc6, 
    0x6b, 0xad, 0xc3, 0x18, 0xa4, 0x91, 0x5a, 0x6b, 
    0xc1, 0xa6, 0x69, 0xa4, 0xda, 0x5a, 0xd7, 0xb2, 
    0x93, 0x68, 0x36, 0xad, 0x19, 0x73, 0x4a, 0x1b, 
    0x63, 0x6e, 0x3b, 0x4e, 0xb5, 0x66, 0x5c, 0xa9, 
    0x36, 0x6c, 0x89, 0xb4, 0x54, 0x73, 0x36, 0xd5, 
    0xa6, 0xc6, 0xf6, 0x31, 0x44, 0x8d, 0x5a, 0x53, 
    0x58, 0xdb, 0x9b, 0x6a, 0xe2, 0x62, 0xcd, 0x36, 
    0xa3, 0x22, 0x1b, 0x63, 0xc5, 0x35, 0xad, 0xb0, 
    0x75, 0xb0, 0xa3, 0xa5, 0x5a, 0x73, 0x35, 0xa7, 
    0x69, 0xad, 0x0b, 0x0f, 0xd5, 0xc0, 0xcd, 0x36, 
    0x5a, 0x6b, 0x39, 0xa6, 0x2b, 0xab, 0xf4, 0x5b, 
    0x80, 0x0f, 0x00, 0x00, 0xe0, 0x53, 0x6e, 0x14, 
    0xd2, 0xf1, 0xc9, 0x82, 0x52, 0xf6, 0xdc, 0xc4, 
    0x5a, 0xe1, 0x6e, 0x28, 0xfe, 0x30, 0xa3, 0xbb, 
    0x56, 0x83, 0x82, 0x4c, 0x37, 0xf6, 0x12, 0x29, 
    0xd7, 0x87, 0x38, 0x1f, 0xec, 0x55, 0xfa, 0x1c, 
    0xcc, 0x1e, 0x5e, 0x30, 0x2d, 0x00, 0x4a, 0x67, 
    0x45, 0x38, 0xe4, 0xcf, 0xeb, 0x44, 0x32, 0x78, 
    0x34, 0xa4, 0x1b, 0x0e, 0x35, 0x34, 0x63, 0xdf, 
    0xa6, 0xb2, 0x92, 0xba, 0xfc, 0x7d, 0x4d, 0xad, 
    0xad, 0x53, 0xba, 0xe5, 0x30, 0x52, 0xbf, 0x31, 
    0x39, 0x57, 0x42, 0x9f, 0xc0, 0x80, 0x89, 0x1c, 
    0x39, 0xed, 0x7b, 0xb7, 0x1c, 0x5a, 0xfc, 0x9d, 
    0xbe, 0xe5, 0x14, 0x7e, 0xd5, 0xad, 0xc8, 0x65, 
    0xb7, 0x70, 0x3e, 0x83, 0x1a, 0x6c, 0x3c, 0xc4, 
    0x35, 0x9e, 0xc8, 0x36, 0xdc, 0xa4, 0x09, 0xc9, 
    0x07, 0x1e, 0xe3, 0x68, 0x0b, 0x88, 0x5e, 0x24, 
    0x9e, 0x00, 0x7e, 0x09, 0xd6, 0xc5, 0x2b, 0x06, 
    0x89, 0xb5, 0x8a, 0x48, 0x0d, 0xae, 0x81, 0x9e, 
    0x18, 0xad, 0x36, 0xf6, 0xa1, 0xf0, 0x70, 0x7d, 
    0x1d, 0x57, 0x53, 0x02, 0xe1, 0xbd, 0xc7, 0xf2, 
    0x85, 0x43, 0xf1, 0xdc, 0xdf, 0x06, 0x18, 0xb3, 
    0x2a, 0xc7, 0x2b, 0x0d, 0xd6, 0x18, 0xad, 0xbd, 
    0x9a, 0x82, 0x35, 0x5b, 0x73, 0xad, 0xcc, 0x5a, 
    0xb6, 0xb5, 0x1b, 0xd5, 0x70, 0xb5, 0x94, 0xb2, 
    0xd8, 0x46, 0x15, 0xf9, 0x5a, 0x8c, 0xd9, 0xd6, 
    0x25, 0xad, 0xf4, 0x9a, 0xad, 0xa9, 0x48, 0x6b, 
    0xd6, 0x8b, 0xc4, 0xca, 0x02, 0xdd, 0xc4, 0x3a, 
    0x1a, 0x81, 0x92, 0xc6, 0x1b, 0xcd, 0xe7, 0x5a, 
    0xb6, 0x31, 0xad, 0x31, 0x32, 0xb4, 0x9d, 0xd6, 
    0x10, 0x07, 0xa4, 0x91, 0xad, 0xd9, 0x9a, 0x62, 
    0xb5, 0xd6, 0x82, 0xad, 0x83, 0x4d, 0x62, 0x49, 
    0xad, 0x29, 0x9a, 0x6d, 0x29, 0xa7, 0x60, 0xbd, 
    0x00, 0xf8, 0x05, 0x00, 0x00, 0x3a, 0xec, 0x5b, 
    0x6c, 0xeb, 0x2f, 0x28, 0xed, 0x61, 0x82, 0x7e, 
    0xee, 0x10, 0x49, 0x40, 0xd0, 0xda, 0xe3, 0x4a, 
    0xf7, 0x9f, 0xac, 0x7f, 0xbf, 0x3c, 0x08, 0xbe, 
    0x86, 0x93, 0x57, 0xe7, 0xb2, 0x84, 0xf7, 0xda, 
    0xa1, 0xf0, 0x4a, 0x76, 0xb0, 0xb7, 0xae, 0x9e, 
    0x40, 0xb6, 0x84, 0x2f, 0xb4, 0xd0, 0x2d, 0x08, 
    0x6a, 0xe5, 0x06, 0x0e, 0xc0, 0xd2, 0x74, 0x1c, 
    0x1a, 0x7b, 0xd3, 0x16, 0xd7, 0xbc, 0x83, 0xd8, 
    0x53, 0x78, 0xe4, 0x46, 0xf2, 0x8f, 0x59, 0x3d, 
    0x14, 0xde, 0x3e, 0x9f, 0x16, 0x49, 0xf8, 0x49, 
    0x5d, 0xad, 0xe3, 0x1c, 0x44, 0xeb, 0x5e, 0xfd, 
    0x2d, 0xe1, 0xcc, 0xc4, 0x79, 0x73, 0x8d, 0x58, 
    0x87, 0x7e, 0x00, 0x94, 0xb6, 0xe4, 0x85, 0x5c, 
    0x67, 0x84, 0x7a, 0x8d, 0xfb, 0x3f, 0x43, 0x06, 
    0xd0, 0xf3, 0x3d, 0xac, 0x46, 0x80, 0xcb, 0x5a, 
    0xc1, 0xc9, 0x49, 0xf5, 0x9d, 0xaa, 0x18, 0x57, 
    0x38, 0x07, 0x9b, 0x65, 0xf8, 0xbd, 0x2f, 0xfa, 
    0x3a, 0xad, 0x34, 0xfc, 0x5c, 0xb7, 0x0f, 0x47, 
    0xc2, 0xba, 0x19, 0xba, 0xca, 0x99, 0xb5, 0x3a, 
    0xd2, 0x2e, 0x26, 0x89, 0xcd, 0xb5, 0x54, 0x8b, 
    0x35, 0x43, 0x6b, 0xca, 0x1b, 0x4e, 0x8d, 0x58, 
    0x5b, 0x53, 0x05, 0x36, 0xb9, 0xa2, 0x69, 0x1a, 
    0x6b, 0x4d, 0x42, 0x5c, 0xa7, 0xaa, 0x4f, 0x39, 
    0xd7, 0x7a, 0x6a, 0x29, 0x61, 0x6b, 0x36, 0x0b, 
    0xb3, 0x0d, 0x62, 0x5b, 0xab, 0xd9, 0x91, 0x3c, 
    0x6d, 0x44, 0x63, 0x84, 0xc6, 0x48, 0x75, 0xb2, 
    0x48, 0x6d, 0xd9, 0xd6, 0x64, 0xcd, 0x82, 0x1c, 
    0xa2, 0xa8, 0xca, 0x29, 0x16, 0x4a, 0xd2, 0x8d, 
    0x6b, 0x29, 0xc7, 0x1a, 0xcd, 0x35, 0x5a, 0x74, 
    0x54, 0xa7, 0x29, 0x42, 0x6b, 0xb6, 0xa8, 0x9a, 
    0xa1, 0x31, 0x03, 0x81, 0x00, 0x96, 0x2a, 0x60, 
};

/*****************************************************************************
 * aout_SpdifThread: audio output thread that sends raw spdif data
 *                   to an external decoder
 *****************************************************************************
 * This output thread is quite specific as it can only handle one fifo.
 * Now fifo #0 is hardcoded, but when the output is launched by the decoder,
 * we'll be able to associate the related fifo to this output.
 *
 * Note: all this is wrong: spdif can demux up to 8 ac3 streams, and can
 * even take care of time stamps (cf ac3 spec).
 *****************************************************************************/
void aout_SpdifThread( aout_thread_t * p_aout )
{
    u8 *        pi_blank;
    boolean_t   b_have_data;
    mtime_t     date;

    /* when the output is launched from the decoder,
     * we won't have to use a hardcoded value for frame size */
    pi_blank = malloc( 6144/*p_aout->fifo[0].l_frame_size*/ );
    memset( pi_blank, 0, sizeof(pi_blank) );
    memcpy( pi_blank, pi_spdif_blank, sizeof(pi_spdif_blank) );

    /* the thread wait until there is some data is aout fifo */
    vlc_mutex_lock( &p_aout->fifo[0].data_lock);
    vlc_cond_wait( &p_aout->fifo[0].data_wait,
                   &p_aout->fifo[0].data_lock);
    vlc_mutex_unlock (&p_aout->fifo[0].data_lock);
        
    date = mdate();
    while( !p_aout->b_die )
    {
        /* we leave some time for aout fifo to fill */
        msleep( 20000 );

        vlc_mutex_lock( &p_aout->fifo[0].data_lock );
        b_have_data = !AOUT_FIFO_ISEMPTY( p_aout->fifo[0] );
        vlc_mutex_unlock (&p_aout->fifo[0].data_lock);

        if( b_have_data )
        {
            /* play spdif frame to the external decoder */
            date = mdate();
            p_aout->pf_play( p_aout,
                             p_aout->fifo[0].buffer +
                             p_aout->fifo[0].l_frame_size *
                                        p_aout->fifo[0].l_start_frame,
                             p_aout->fifo[0].l_frame_size );

            vlc_mutex_lock( &p_aout->fifo[0].data_lock);
            p_aout->fifo[0].l_start_frame = 
                (p_aout->fifo[0].l_start_frame + 1 ) & AOUT_FIFO_SIZE;
            vlc_mutex_unlock (&p_aout->fifo[0].data_lock);
        }
        else
        {
            intf_WarnMsg( 2, "spdif warning: waiting (%lld)",
                p_aout->fifo[0].date[p_aout->fifo[0].l_start_frame-1] - date );

            date = mdate();
            /* insert blank frame for stream continuity to
             * the external decoder */
            p_aout->pf_play( p_aout, pi_blank, p_aout->fifo[0].l_frame_size );
        }
    }

    return;
}

